!  3.3 ׂ@
program power_method
   implicit none
   real a(4,4),x(4),emax
   real, parameter :: eps = 1e-14
   integer ind
   a(1,1) = 14;    a(1,2) = 7;     a(1,3) = 3;     a(1,4) = -5
   a(2,1) = 7;     a(2,2) = 13;    a(2,3) = -2;    a(2,4) = -1
   a(3,1) = 3;     a(3,2) = -2;    a(3,3) = 6;     a(3,4) = 1
   a(4,1) = -5;    a(4,2) = -1;    a(4,3) = 1;     a(4,4) = 8
   call max_eigenvalue(a,4,eps,emax,x,ind)
   if (ind < 0) then
      print *,'No Convergence max_eigenvalue !'
    else
      call check_eigenvalue(a,emax,x,4)
   endif
end program power_method

subroutine max_eigenvalue(a,n,eps,emax,x,ind)
   implicit none
   real a(n,n),x(n),eps,emax,y1(n),y2(n)
   real p1,p2,dp,e1,e2
   integer, parameter :: itmax = 1000
   integer ind,i,j,n,it
   call random_number(y1(1:n))
   e1 = 0
   do it = 1, itmax
      do i = 1, n
         y2(i) = 0
         do j = 1, n
            y2(i) = y2(i) + a(i,j)*y1(j)
         enddo
      enddo
      p1 = 0;   p2 = 0
      do i = 1, n
         p1 = p1 + y1(i)*y2(i)
         p2 = p2 + y2(i)*y2(i)
      enddo
      e2 = p2/p1
      p2 = 1/sqrt(p2)
      if (e2 < 0) p2 = -p2
      dp = 0
      do i = 1, n
         p1 = p2*y2(i)
         dp = dp + abs(y1(i)-p1)
         y1(i) = p1
      enddo
      if (abs((e1-e2)/e2) < eps .and. dp < eps) exit
      e1 = e2
   enddo
   if (it > itmax) then
      ind = -itmax
      return
   endif
   ind = it
   emax = e2
   x(1:n) = y1(1:n)
end subroutine max_eigenvalue

subroutine check_eigenvalue(a,eigen,x,n)
   implicit none
   real a(n,n),eigen,x(n),xe,err
   integer n,i,j
   character(80) form
   err = 0
   do i = 1, n
      xe = 0
      do j = 1, n
         xe = xe + a(i,j)*x(j)
      enddo
      err = max(err,abs(eigen*x(i)-xe))
   enddo
   print "('  Eigenvalue  = ',f14.7,'   Error = ',es11.4)",eigen,err
   form = "('  Eigenvector = (',f12.5,@(',',f12.5),')')"
   i = index(form,'@')
   form(i:i) = char(ichar('0')+n-1)
   print form,x(1),(x(i),i=2,n)
end subroutine check_eigenvalue
