!  3.4 tׂ@
program invpower_method
   real a(4,4),x(4),emax
   real, parameter :: eps = 1e-14
   integer ind
   a(1,1) = 14;    a(1,2) = 7;     a(1,3) = 3;     a(1,4) = -5
   a(2,1) = 7;     a(2,2) = 13;    a(2,3) = -2;    a(2,4) = -1
   a(3,1) = 3;     a(3,2) = -2;    a(3,3) = 6;     a(3,4) = 1
   a(4,1) = -5;    a(4,2) = -1;    a(4,3) = 1;     a(4,4) = 8
   call min_eigenvalue(a,4,eps,emax,x,ind)
   if (ind < 0) then
      print *,'No Convergence min_eigenvalue !'
    else
      call check_eigenvalue(a,emax,x,4)
   endif
end program invpower_method

subroutine min_eigenvalue(a,n,eps,emin,x,ind)
   implicit none
   real a(n,n),x(n),eps,emin
   real p1,p2,dp,e1,e2
   integer, parameter :: itmax = 1000
   integer n,ind,i,it
   real, allocatable :: a1(:,:),b(:),y1(:),y2(:)
   integer, allocatable :: ipv(:)
   allocate ( a1(n,n),b(n),y1(n),y2(n),ipv(n) )
   a1(:,:) = a(:,:)
   call random_number(y1(1:n))
   call ludecomposition(a1,n,ipv,p1)
   e1 = 1d100
   do it = 1, itmax
      b(:) = y1(:)
      call lusolution(a1,ipv,b,n,y2)
      p1 = 0;   p2 = 0
      do i = 1, n
         p1 = p1 + y1(i)*y2(i)
         p2 = p2 + y2(i)*y2(i)
      enddo
      e2 = p1/p2
      p2 = 1/sqrt(p2)
      if (e2 < 0) p2 = -p2
      dp = 0
      do i = 1, n
         p1 = p2*y2(i)
         dp = dp + abs(y1(i)-p1)
         y1(i) = p1
      enddo
      if (abs((e1-e2)/e2) < eps .and. dp < eps) exit
      e1 = e2
   enddo
   emin = e2
   x(1:n) = y1(1:n)
   deallocate ( a1,b,y1,y2,ipv )
   if (it > itmax) then
      ind = -itmax
    else
      ind = it
   endif
end subroutine min_eigenvalue

subroutine check_eigenvalue(a,eigen,x,n)
   implicit none
   real a(n,n),eigen,x(n),xe,err
   integer n,i,j
   character(80) form
   err = 0
   do i = 1, n
      xe = 0
      do j = 1, n
         xe = xe + a(i,j)*x(j)
      enddo
      err = max(err,abs(eigen*x(i)-xe))
   enddo
   print "('  Eigenvalue  = ',f14.7,'   Error = ',es11.4)",eigen,err
   form = "('  Eigenvector = (',f12.5,@(',',f12.5),')')"
   i = index(form,'@')
   form(i:i) = char(ichar('0')+n-1)
   print form,x(1),(x(i),i=2,n)
end subroutine check_eigenvalue

subroutine ludecomposition(a,n,ipv,det)
   implicit none
   real a(n,n),det,dd,amax
   integer n,ipv(n),i,j,k,imax,ip
   do i = 1, n
      ipv(i) = i
   enddo
   ip = 1
   do k = 1, n-1
      amax = a(ipv(k),k)
      imax = k
      do i = k+1, n
         if (amax < abs(a(ipv(i),k))) then
            amax = abs(a(ipv(i),k))
            imax = i
         endif
      enddo
      if (imax /= k) then
         j = ipv(k)
         ipv(k) = ipv(imax)
         ipv(imax) = j
         ip = -ip
      endif
      do i = k+1, n
         dd = a(ipv(i),k)/a(ipv(k),k)
         do j = k+1, n
            a(ipv(i),j) = a(ipv(i),j) - dd*a(ipv(k),j)
         enddo
         a(ipv(i),k) = dd
      enddo
   enddo
   det = a(ipv(1),1)
   do i = 2, n
      det = det*a(ipv(i),i)
   enddo
   det = det*ip
end subroutine ludecomposition

subroutine lusolution(a,ipv,b,n,x)
   implicit none
   real a(n,n),b(n),x(n),dd
   integer n,ipv(n),i,j
   do j = 1, n-1
      do i = j+1, n
         b(ipv(i)) = b(ipv(i)) - a(ipv(i),j)*b(ipv(j))
      enddo
   enddo
   x(n) = b(ipv(n))/a(ipv(n),n)
   do i = n-1, 1, -1
      dd = b(ipv(i))
      do j = i+1, n
         dd = dd - a(ipv(i),j)*x(j)
      enddo
      x(i) = dd/a(ipv(i),i)
   enddo
end subroutine lusolution
