!  3.5 Rr@
program Jacobian_method
   implicit none
   integer, parameter :: n0 = 5
   real a(n0,n0),x(n0,n0),eigen(n0)
   real, parameter :: eps = 1e-15
   integer ind,i,j,k
   a(1,1) = 1;  a(1,2) = 1;  a(1,3) = 1;  a(1,4) = 1;  a(1,5) = 1
                a(2,2) = 2;  a(2,3) = 2;  a(2,4) = 2;  a(2,5) = 2
                             a(3,3) = 3;  a(3,4) = 3;  a(3,5) = 3
                                          a(4,4) = 4;  a(4,5) = 4
                                                       a(5,5) = 5
   do j = 1, n0-1
      do i = j+1, n0
         a(i,j) = a(j,i)
      enddo
   enddo
   call jacobian_eigen(a,n0,eps,eigen,x,ind)
   do k = 1, n0
      call check_eigenvalue(a,eigen(k),x(1,k),n0)
   enddo
end program Jacobian_method

subroutine jacobian_eigen(a,n,eps,eigen,x,ind)
   implicit none
   real a(n,n),eps,eigen(n),x(n,n)
   real amax,c1,c2,tt,cc,ss,sc
   integer, parameter :: itmax = 1000
   integer ind,i,j,n,it,p,q
   real, allocatable :: a1(:,:),am(:)
   integer, allocatable :: im(:)
   allocate ( a1(n,n),am(n),im(n) )
   a1(:,:) = a(:,:)
   do j = 2, n
      am(j) = abs(a1(1,j));     im(j) = 1
      do i = 2, j-1
         if (abs(a1(i,j)) > am(j)) then
            am(j) = abs(a1(i,j))
            im(j) = i
         endif
      enddo
   enddo
   x(:,:) = 0
   do i = 1, n
      x(i,i) = 1
   enddo
   do it = 1, itmax
      amax = 0
      do j = 2, n
         if (am(j) > amax) then
            amax = am(j)
            q = j
         endif
      enddo
      if (amax < eps) exit
      p = im(q)
      tt = (a1(q,q)-a1(p,p))/(2*a1(p,q))
      if (tt >= 0) then
         tt = 1/(tt + sqrt(tt*tt+1))
       else
         tt = 1/(tt - sqrt(tt*tt+1))
      endif
      cc = 1/sqrt(1+tt*tt);   ss = cc*tt
      sc = ss/(1+cc)
      do i = 1, n
         if (i == p) then
            a1(p,p) = a1(p,p) - a1(p,q)*tt
            a1(q,q) = a1(q,q) + a1(p,q)*tt
            a1(p,q) = 0;       a1(q,p) = 0
          else if (i /= q) then
            c1 = a1(i,p) - ss*(a1(i,q) + sc*a1(i,p))
            c2 = a1(i,q) + ss*(a1(i,p) - sc*a1(i,q))
            a1(i,p) = c1;     a1(i,q) = c2
            a1(p,i) = c1;     a1(q,i) = c2
         endif
      enddo
      do i = 1, n
         c1 = x(i,p) - ss*(x(i,q) + sc*x(i,p))
         c2 = x(i,q) + ss*(x(i,p) - sc*x(i,q))
         x(i,p) = c1;     x(i,q) = c2
      enddo
      do j = 2, n
         if (j == p .or. j == q) then
            am(j) = abs(a1(1,j));     im(j) = 1
            do i = 2, j-1
               if (abs(a1(i,j)) > am(j)) then
                  am(j) = abs(a1(i,j))
                  im(j) = i
               endif
            enddo
          else
            if (am(j) < abs(a1(p,j))) then
               am(j) = abs(a1(p,j))
               im(j) = p
              else if (am(j) < abs(a1(q,j))) then
               am(j) = abs(a1(q,j))
               im(j) = q
            endif
         endif
      enddo
   enddo
   do i = 1, n
      eigen(i) = a1(i,i)
   enddo
   deallocate ( a1,am,im )
   if (it > itmax) then
      ind = -itmax
    else
      ind = it
   endif
end subroutine jacobian_eigen

subroutine check_eigenvalue(a,eigen,x,n)
   implicit none
   real a(n,n),eigen,x(n),xe,err
   integer n,i,j
   character(80) form
   err = 0
   do i = 1, n
      xe = 0
      do j = 1, n
         xe = xe + a(i,j)*x(j)
      enddo
      err = max(err,abs(eigen*x(i)-xe))
   enddo
   print "('  Eigenvalue  = ',f14.7,'   Error = ',es11.4)",eigen,err
   form = "('  Eigenvector = (',f12.5,@(',',f12.5),')')"
   i = index(form,'@')
   form(i:i) = char(ichar('0')+n-1)
   print form,x(1),(x(i),i=2,n)
end subroutine check_eigenvalue
