!  3.7 2@ɂ3dΊpšŗLlvZC
!      ыtׂ@ɂŗLl̐xƌŗLxNǧvZ
program Hauseholder_Eigen
   implicit none
   integer, parameter :: n0 = 5
   real a(n0,n0),al(n0),be(n0),eigen(n0),x(n0,n0)
   real, parameter :: eps = 1e-5, eps1 = 1e-14
   integer k,ind
   a(1,1) = 1;  a(1,2) = 1;  a(1,3) = 1;  a(1,4) = 1;  a(1,5) = 1
                a(2,2) = 2;  a(2,3) = 2;  a(2,4) = 2;  a(2,5) = 2
                             a(3,3) = 3;  a(3,4) = 3;  a(3,5) = 3
                                          a(4,4) = 4;  a(4,5) = 4
                                                       a(5,5) = 5
   call householder(a,n0,al,be)
   call root_bisection(al,be,n0,eps,eigen,ind)
   do k = 1, n0
      call min_eigenvalue_tri(al,be,eigen(k),n0,eps1,x(1,k),ind)
      call back_htransform(x(1,k),a,n0)
      call check_eigenvalue_sym(a,eigen(k),x(1,k),n0)
   enddo
end program Hauseholder_Eigen

subroutine root_bisection(alpha,beta,n,eps,eigen,ind)
   implicit none
   real alpha(n),beta(n),eps,eigen(n),r1,r2,a,b,c,g
   integer n,ind,i,k,it,itm,nc,ip
   integer, parameter :: itmax = 1000
   r2 = abs(alpha(1))+abs(beta(1))
   do i = 2, n-1
      r2 = max(r2,abs(beta(i-1))+abs(alpha(i))+abs(beta(i)))
   enddo
   r2 = max(r2,abs(beta(n-1))+abs(alpha(n)))
   r1 = -r2
   itm = 0
   do k = 1, n
      a = r1;  b = r2
      do it = 1, itmax
         c = (a+b)/2
         g = c - alpha(1)
         nc = 0;     ip = 0
         if (g <= 0) nc = 1
         do i = 2, n
            if (ip == 1) then
               g = c - alpha(i)
             else if (abs(g) < 1e-15) then
               ip = 1
               cycle
             else
               g = c - alpha(i) - beta(i-1)**2/g
            endif
            if (g <= 0) nc = nc + 1
            ip = 0
         enddo
         if (nc >= n .and. r1 < a) then
            r1 = a
         endif
         if (nc >= k) then
            a = c
          else
            b = c
         endif
         if (b-a < eps) exit
      enddo
      itm = max(it,itm)
      eigen(k) = c
      r2 = c
   enddo
   if (itm > itmax) then
      ind = -itmax
    else
      ind = itm
   endif
end subroutine root_bisection

subroutine min_eigenvalue_tri(alpha,beta,eigen,n,eps,x,ind)
   implicit none
   real alpha(n),beta(n),eigen,eps,x(n)
   real y1(n),y2(n),a1(n),b1(n),c1(n),d1(n),y(n)
   real p1,p2,dp,e1,e2
   integer n,ind,i,it,ip(n)
   integer, parameter :: itmax = 1000
   b1(1) = alpha(1) - eigen
   c1(1) = beta(1)
   do i = 2, n
      a1(i) = beta(i-1)
      b1(i) = alpha(i) - eigen
      c1(i) = beta(i)
   enddo
   call random_number(y1(1:n))
   e1 = 1d100
   call ludecomposition3(a1,b1,c1,d1,ip,n)
   do it = 1, itmax
      y = y1
      call lusolution3(a1,b1,c1,d1,ip,y,n,y2)
      p1 = 0;   p2 = 0
      do i = 1, n
         p1 = p1 + y1(i)*y2(i)
         p2 = p2 + y2(i)*y2(i)
      enddo
      e2 = p1/p2
      p2 = 1/sqrt(p2)
      if (e2 < 0) p2 = -p2
      dp = 0
      do i = 1, n
         p1 = p2*y2(i)
         dp = dp + abs(y1(i)-p1)
         y1(i) = p1
      enddo
      if (abs((e1-e2)/e2) < eps .and. dp < eps) exit
      e1 = e2
   enddo
    eigen = eigen + e2
    x(1:n) = y1(1:n)
   if (it > itmax) then
      ind = -itmax
    else
      ind = it
   endif
end subroutine min_eigenvalue_tri

subroutine ludecomposition3(a,b,c,d,ip,n)
   implicit none
   real a(n),b(n),c(n),d(n),dd,cc
   integer n,k,ip(n)
   d(1) = 0
   do k = 2, n
      d(k) = 0
      if (abs(a(k)) <= abs(b(k-1))) then
         dd = -a(k)/b(k-1)
         b(k) = b(k) + dd*c(k-1)
         a(k) = dd
         ip(k) = 0
       else
         cc = c(k-1)
         dd = -b(k-1)/a(k)
         b(k-1) = a(k)
         c(k-1) = b(k)
         d(k-1) = c(k)
         a(k) = dd
         b(k) = cc + dd*b(k)
         c(k) = dd*c(k)
         ip(k) = 1
      endif
   enddo
end subroutine ludecomposition3

subroutine lusolution3(a,b,c,d,ip,y,n,x)
   implicit none
   real a(n),b(n),c(n),d(n),y(n),x(n),yy
   integer n,k,ip(n)
   do k = 2, n
      if (ip(k) == 0) then
         y(k) = y(k) + a(k)*y(k-1)
       else
         yy = y(k)
         y(k) = y(k-1) + a(k)*yy
         y(k-1) = yy
      endif
   enddo
   x(n) = y(n)/b(n)
   x(n-1) = (y(n-1) - c(n-1)*x(n))/b(n-1)
   do k = n-2, 1, -1
      x(k) = (y(k) - c(k)*x(k+1) - d(k)*x(k+2))/b(k)
   enddo
end subroutine lusolution3

subroutine check_eigenvalue_sym(a,eigen,x,n)
   implicit none
   real a(n,n),eigen,x(n),xe,err
   integer n,i,j
   character(80) form
   err = 0
   do i = 1, n
      xe = a(i,i)*x(i)
      do j = 1, i-1
         xe = xe + a(j,i)*x(j)
      enddo
      do j = i+1, n
         xe = xe + a(i,j)*x(j)
      enddo
      err = max(err,abs(eigen*x(i)-xe))
   enddo
   print "('  Eigenvalue  = ',f14.7,'   Error = ',es11.4)",eigen,err
   form = "('  Eigenvector = (',f12.5,@(',',f12.5),')')"
   i = index(form,'@')
   form(i:i) = char(ichar('0')+n-1)
   print form,x(1),(x(i),i=2,n)
end subroutine check_eigenvalue_sym

subroutine householder(a,n,alpha,beta)
   implicit none
   real a(n,n),alpha(n),beta(n)
   real s,c,d
   integer i,j,n,k,k1
   real, allocatable :: a1(:,:),u(:),p(:)
   allocate ( a1(n,n),u(n),p(n) )
   a1(:,:) = a(:,:)
   do j = 1, n
      a1(j,j) = a(j,j)
      do i = 1, j-1
         a1(i,j) = a(i,j)
         a1(j,i) = a(i,j)
      enddo
   enddo
   do k = 1, n-2
      k1 = k + 1
      s = 0
      do i = k1, n
         u(i) = a1(i,k)
         s = s + u(i)*u(i)
      enddo
      s = sqrt(s)
      alpha(k) = a1(k,k)
      if (s == 0) then
         beta(k)  = 0
         do j = k1, n
            a1(j,k) = 0
         enddo
         cycle
      endif
      if (u(k1) < 0) s = -s
      u(k1) = u(k1) + s
      beta(k)  = -s
      c = 1/sqrt(u(k1)*s)
      u(k1:n) = u(k1:n)*c
      do i = k1, n
         p(i) = 0
         do j = k1, n
            p(i) = p(i) + a1(i,j)*u(j)
         enddo
      enddo
      d = 0
      do i = k1, n
         d = d + p(i)*u(i)
      enddo
      d = d/2
      do i = k1, n
         p(i) = p(i) - d*u(i)
      enddo
      do j = k1, n
         a1(j,k) = u(j)
         do i = k1, n
            a1(i,j) = a1(i,j) - (u(i)*p(j) + u(j)*p(i))
         enddo
      enddo
   enddo
   alpha(n-1) = a1(n-1,n-1)
   beta(n-1)  = a1(n,n-1)
   alpha(n)   = a1(n,n)
   do j = 1, n-1
      do i = j+1, n
         a(i,j) = a1(i,j)
      enddo
   enddo
   deallocate ( a1,u,p )
end subroutine householder

subroutine back_htransform(x,a,n)
   implicit none
   real x(n),a(n,n)
   real p1
   integer n,i,j
   do j = n-2, 1, -1
      p1 = 0
      do i = j+1, n
         p1 = p1 + a(i,j)*x(i)
      enddo
      do i = j+1, n
         x(i) = x(i) - p1*a(i,j)
      enddo
   enddo
end subroutine back_htransform
