!  4.6 G~[gEKEXϕ@\SԂ̐ϕ\
!      W쐬pTu[`
program Hermite_Coefficients
   implicit none
   integer, parameter :: n = 7, itmax = 30
   real heposition(n),heweight(n)
   real, parameter :: pi = 3.141592653589793, eps = 1e-25
   real(16) x,xx,p,dp,w,x1,x2,x3,fac
   integer k,it,n2,nodd
   n2 = (n+1)/2;     nodd = mod(n,2)
   fac = 0.5
   do k = 1, n
      fac = 2*fac*k
   enddo
   do k = 1, n2
      if (k < 4) then
         if (nodd == 0) then
            x = (2*k-1)*pi/(2*sqrt(2*n+1.0))
          else
            x = (k-1)*pi/sqrt(2*n+1.0)
         endif
       else
         x  = 3*(x3-x2)+x1
         x1 = x2
         x2 = x3
         x3 = x
      endif
      do it = 1, itmax
         call hermite(n,x,p,dp)
         xx = x - p/dp
         if (abs(xx-x) < eps .or. abs(p) < eps) exit
         x = xx
      enddo
      if (it > itmax) then
         print *,'No convergence at k = ',k
         stop
      endif
      call hermite(n-1,x,w,dp)
      w = fac/(n*w)**2
      if (k == 1) then
         x1 = x
       else if (k == 2) then
         x2 = x
       else
         x3 = x
      endif
      heposition(k) = x
      heweight(k) = w*sqrt(pi)
   enddo
   open(10,file='hermite.inc')
   write(10,*) '   integer, parameter :: narray =',n2
   call output_array(10,heposition,n2,'heposition(narray)')
   call output_array(10,heweight,n2,'heweight(narray)')
end program Hermite_Coefficients

subroutine hermite(n,x,p,dp)
   implicit none
   real(16) x,p,dp,p0,p1,p2
   integer k,n
   p0 = 1
   p1 = 2*x
   do k = 2, n
      p2 = 2*(x*p1 - (k-1)*p0)
      p0 = p1
      p1 = p2
   enddo
   p  = p1
   dp = 2*n*p0
end subroutine hermite

subroutine output_array(ndev,array,nd,chd)
   implicit none
   real array(*)
   integer ndev,nd,n4,n0,k
   character chd*(*)
   n4 = mod(nd+3,4) + 1
   n0 = nd - n4
   write(ndev,*) '   real :: ',trim(chd),' = (/ &'
   if (n0 > 0) write(ndev,600) (array(k), k=1,n0)
   if (n4 == 1) then
      write(ndev,610) array(n0+1)
    else if (n4 == 2) then
      write(ndev,620) array(n0+1),array(n0+2)
    else if (n4 == 3) then
      write(ndev,630) array(n0+1),array(n0+2),array(n0+3)
    else
      write(ndev,640) array(n0+1),array(n0+2),array(n0+3),array(n0+4)
   endif
600 format('     ',4(es22.15,', '),' &')
610 format('     ',es22.15,' /)')
620 format('     ',es22.15,', ',es22.15,' /)')
630 format('     ',2(es22.15,', '),es22.15,' /)')
640 format('     ',3(es22.15,', '),es22.15,' /)')
end subroutine output_array
