!  5.4 L֐ߎ
program rational_approximation_test
   implicit none
   integer, parameter :: n0 = 10
   real xp(0:n0),yp(0:n0)
   real x,y,z,h
   integer i,n,num,den
   num = 4
   den = n0 - num
   xp(0) = -2
   xp(n0) = 2
   h = (xp(n0)-xp(0))/n0
   do i = 0, n0
      xp(i) = h*i + xp(0)
      yp(i) = 1/sqrt(49*xp(i)**2 + 1)
   enddo
   call rational_approximation(x,y,xp,yp,num,den,1)
   n = 200
   h = (xp(n0)-xp(0))/n
   do i = 0, n
      x = h*i + xp(0)
      call rational_approximation(x,y,xp,yp,num,den,0)
      z = 1/sqrt(49*x*x + 1)
      print *,x,y,z
   enddo
end program rational_approximation_test

subroutine rational_approximation(x,y,xp,yp,nm,nd,mode)
   implicit none
   real x,y,xp(0:nm+nd),yp(0:nm+nd),num,den,xx
   integer nm,nd,mode,n,i,na
   real, allocatable, save :: bp(:),rp(:)
   real, allocatable :: cf(:),cg(:),ap(:)
   real, save :: x0,xmin,xmax
   integer, save:: nb,nr
   if (mode == 1) then
      n = nm + nd
      if (allocated(bp)) deallocate ( bp, rp )
      allocate ( cf(0:n), cg(0:n+1), ap(0:n) )
      allocate ( bp(0:n), rp(0:n) )
      call make_dd_polynomials(xp,yp,n,cf,cg)
      call polynom_eucledean(cg,n+1,cf,n,nm,ap,na,bp,nb,rp,nr)
      deallocate ( cf, cg, ap )
      x0   = (xp(0)+xp(n))/2
      xmin = xp(0)
      xmax = xp(n)
      return
   endif 
   if (x < xmin) then
      print *,'Out of bounds -- rational_approximation',x
    else if (x > xmax) then
      print *,'Out of bounds -- rational_approximation',x
   endif
   num = 0
   xx  = x - x0
   do i = nr, 0, -1
      num = num*xx + rp(i)
   enddo
   den = 0
   do i = nb, 0, -1
      den = den*xx + bp(i)
   enddo
   y = num/den
end subroutine rational_approximation

subroutine make_dd_polynomials(xp,yp,n,gp,om)
   implicit none
   real xp(0:n),yp(0:n),gp(0:n),om(0:n+1),dd1(n),dd2(n),dd(0:n),x0
   integer n,np,i,j
   do i = 1, n
      dd1(i) = (yp(i-1)-yp(i))/(xp(i-1)-xp(i))
   enddo
   dd(0) = yp(0)
   dd(1) = dd1(1)
   do j = 1, n-1
      do i = 1, n-j
         dd2(i) = (dd1(i)-dd1(i+1))/(xp(i-1)-xp(i+j))
      enddo
      dd1(1:n-j) = dd2(1:n-j)
      dd(j+1) = dd1(1)
   enddo
   x0 = (xp(0)+xp(n))/2
   gp(0) = dd(n)
   do i = n-1, 0, -1
      np = n-i-1
      gp(1:np+1) = gp(0:np)
      gp(0) = dd(i)
      gp(0:np) = gp(0:np) - (xp(i)-x0)*gp(1:np+1)
   enddo
   om(0) = 1
   do i = 0, n
      om(1:i+1) = om(0:i)
      om(0) = 0
      om(0:i) = om(0:i) - (xp(i)-x0)*om(1:i+1)
   enddo
end subroutine make_dd_polynomials

subroutine polynom_eucledean(p10,n10,p20,n20,m0,ap,na,bp,nb,rp,nr)
   implicit none
   real p10(0:n10),p20(0:n20),ap(0:*),bp(0:*),rp(0:*)
   integer n10,n20,m0,na,nb,nr
   real p1(0:max(n10,n20)),p2(0:max(n10,n20))
   real ps(0:max(n10,n20)),pq(0:max(n10,n20))
   real pm21(0:max(n10,n20)),pm22(0:max(n10,n20))
   integer n1,n2,ns,nq,m11,m12,m21,m22
   p1(:) = p10(:);   n1 = n10
   p2(:) = p20(:);   n2 = n20
   ap(0)   = 1;      m11 = 0
   bp(0)   = 0;      m12 = 0
   pm21(0) = 0;      m21 = 0
   pm22(0) = 1;      m22 = 0
   do while (n1 > m0)
      call poly_div(p1,n1,p2,n2,ps,ns,pq,nq)
      p1(0:n2) = p2(0:n2);      n1 = n2
      p2(0:nq) = pq(0:nq);      n2 = nq
      pq(0:m11) = ap(0:m11);      nq  = m11
      ap(0:m21) = pm21(0:m21);    m11 = m21
      call poly_diff_mult(pq,nq,ps,ns,ap,m11,pm21,m21)
      pq(0:m12) = bp(0:m12);      nq  = m12
      bp(0:m22) = pm22(0:m22);    m12 = m22
      call poly_diff_mult(pq,nq,ps,ns,bp,m12,pm22,m22)
   enddo
   rp(0:n1) = p1(0:n1);       nr = n1
   na = m11;        nb = m12
end subroutine polynom_eucledean

subroutine poly_diff_mult(p1,n1,p2,n2,p3,n3,ps,ns) 
   implicit none
   real p1(0:n1),p2(0:n2),p3(0:n3),ps(0:*)
   integer n1,n2,n3,ns,n0,i,j
   if (n2 == 0 .and. p2(0) == 0) then
      ps(0:n1) = p1(0:n1);      ns = n1
    else if (n3 == 0 .and. p3(0) == 0) then
      ps(0:n1) = p1(0:n1);      ns = n1
    else
      ns = n2 + n3
      ps(0:ns) = 0
      do j = 0, n3
         do i = 0, n2
            ps(i+j) = ps(i+j) + p2(i)*p3(j)
         enddo
      enddo
      n0 = min(n1,ns)
      ps(0:n0) = p1(0:n0) - ps(0:n0)
      if (n1 > n0) then
         ns = n1
         ps(n0+1:n1) = p1(n0+1:n1)
       else if (ns > n0) then
         ps(n0+1:ns) = -ps(n0+1:ns)
      endif
   endif
end subroutine poly_diff_mult

subroutine poly_div(p1,n1,p2,n2,pq,nq,pr,nr) 
   implicit none
   real p1(0:n1),p2(0:n1),pq(0:*),pr(0:*),a,x,pp
   integer n1,n2,nq,nr,i,j
   if (n1 < n2) then
      pq(0) = 0;      nq = 0
      pr(0:n1) = p1(0:n1)
      nr = n1
    else if (n2 == 0) then
      if (p2(0) == 0) then
         print *,'Zero Division -- poly_div'
         pr(0) = 1d100;         nr = 0
        else
         a = 1/p2(0)
         pq(0:n1) = a*p1(0:n1)
         pr(0) = 0;         nr = 0
      endif
    else if (n2 == 1) then
      a = 1/p2(1)
      x = -a*p2(0)
      pp = p1(n1)
      do i = n1-1, 0, -1
         pq(i) = a*pp
         pp = pp*x + p1(i)
      enddo
      nq = n1-1
      pr(0) = pp;      nr = 0
    else
      nq = n1 - n2
      pr(0:n1) = p1(0:n1)
      do j = nq, 0, -1
         a = pr(n2+j)/p2(n2)
         do i = 0, n2-1
            pr(i+j) = pr(i+j) - a*p2(i)
         enddo
         pq(j) = a
      enddo
      nr = n2 - 1
   endif
end subroutine poly_div
