!  8.3 A@ɂ`M`̉@
program thermal_nonlinear
   implicit none
   integer, parameter :: imax = 100
   real T(0:imax),T2(imax),Td(imax),xmax,x,dx,dt,kp,kdt
   real pt(imax),at(imax),bt(imax),ct(imax),del
   real, parameter :: eps = 1e-8
   integer i,nt,ntmax,it,itmax
   kp   = 0.02
   xmax = 10
   dx   = xmax/imax
   dt   = 0.01
   kdt  = kp*dt/(dx*dx)
   ntmax = 1000
   itmax = 100
   do i = 0, imax
      x = dx*i
      if (i < imax/2) then
         T(i) = x
       else
         T(i) = xmax - x
      endif
      write(10,*) i,x,T(i)
   enddo
   do i = 1, imax
      pt(i) = 0.5*kdt*(T(i)**3 + T(i-1)**3)
   enddo
   do nt = 1, ntmax
      do i = 1, imax-1
         Td(i) = T(i) + 0.5*(pt(i)*T(i-1) &
                  - (pt(i)+pt(i+1))*T(i) + pt(i+1)*T(i+1))
      enddo
      do it = 1, itmax                       ! Ԃ[v
         do i = 1, imax-1
            at(i) = -0.5*pt(i)
            bt(i) = 1 + 0.5*(pt(i) + pt(i+1))
            ct(i) = -0.5*pt(i+1)
         enddo
         call tridiagonal_matrix(at,bt,ct,Td,imax-1,T2)
         del = 0
         do i = 1, imax-1
            del = max(del, abs(T(i)-T2(i)))
            T(i) = T2(i)
         enddo
         do i = 1, imax
            pt(i) = 0.5*kdt*(T(i)**3 + T(i-1)**3)
         enddo
         if (del < eps) exit
      enddo
   enddo
   do i = 0, imax
      x = dx*i
      write(11,*) i,x,T(i)
   enddo
   print *,'Stored Initial Data -> 10'
   print *,'Stored Final   Data -> 11'
end program thermal_nonlinear

subroutine tridiagonal_matrix(a,b,c,d,n,x)
   implicit none
   real a(n),b(n),c(n),d(n),x(n)
   real G(n),H(n),den
   integer i,n
   G(1) = -c(1)/b(1)
   H(1) = d(1)/b(1)
   do i = 2, n
      den  = 1/(b(i) + a(i)*G(i-1))
      G(i) = -c(i)*den
      H(i) = (d(i) - a(i)*H(i-1))*den
   enddo
   x(n) = H(n)
   do i = n-1, 1, -1
      x(i) = G(i)*x(i+1) + H(i)
   enddo
end subroutine tridiagonal_matrix
