!  8.9 2|A\̔@3@\ICCG@\
program Poisson_ICCG
   implicit none
   integer, parameter :: nx = 200, ny = 200
   real V(0:nx,0:ny),f(0:nx,0:ny),x,y,dx,dy
   integer i,j,ind
   real, parameter :: pi = 3.141592653589793, eps = 1e-8
   dx = 1.0/nx;      dy = dx
   do j = 0, ny
      y = dy*j
      do i = 0, nx
         x = dx*i
         V(i,j) = 0
         f(i,j) = cos(pi*x)*cos(3*pi*y) + sin(2*pi*x)*sin(4*pi*y)
      enddo
   enddo
   call iccg(V,f,dx,nx,ny,eps,ind)
   print *,'ind = ',ind

   write(10,*) nx,ny,ind
   do j = 0, ny
      do i = 0, nx
         write(10,*) i,j,V(i,j)
      enddo
   enddo
   print *,'Stored Data -> 10'
end program Poisson_ICCG

subroutine iccg(xr,br,h,nx,ny,eps,ind)
   implicit none
   real xr(0:nx,0:ny),br(0:nx,0:ny),h,eps
   real, allocatable :: rr(:,:),pr(:,:),wr(:,:),dg(:,:)
   real del,h2,fa1,fa2,alpha,beta
   integer nx,ny,i,j,it,ind
   integer, parameter :: itmax = 100000
   allocate (rr(nx-1,ny-1),pr(0:nx,0:ny),wr(nx-1,ny-1),dg(nx-1,ny-1))
   pr(:,:) = 0
   h2 = h*h
   do j = 1, ny-1
      do i = 1, nx-1
         rr(i,j) = h2*br(i,j) - (xr(i,j-1) + xr(i-1,j) + xr(i+1,j) &
                                           + xr(i,j+1) - 4*xr(i,j))
      enddo
   enddo
   call ic_solver(rr,wr,dg,nx-1,ny-1,1)
   pr(1:nx-1,1:ny-1) = wr(:,:)
   fa1 = sum(rr(:,:)*wr(:,:))
   do it = 1, itmax
      do j = 1, ny-1
         do i = 1, nx-1
            wr(i,j) = pr(i,j-1) + pr(i-1,j) + pr(i+1,j) &
                                   + pr(i,j+1) - 4*pr(i,j)
         enddo
      enddo
      fa2 = sum(pr(1:nx-1,1:ny-1)*wr(:,:))
      alpha = fa1/fa2
      xr(1:nx-1,1:ny-1) = xr(1:nx-1,1:ny-1)+alpha*pr(1:nx-1,1:ny-1)
      del = 0
      do j = 1, ny-1
         do i = 1, nx-1
            del = max(del, abs(xr(i-1,j)+xr(i+1,j)+xr(i,j-1) &
                               +xr(i,j+1)-4*xr(i,j) - h2*br(i,j)))
         enddo
      enddo
      del = del/h2
!      print *,it,del
      if (del < eps) exit
      rr(:,:) = rr(:,:) - alpha*wr(:,:)
      call ic_solver(rr,wr,dg,nx-1,ny-1,0)
      fa2 = sum(rr(:,:)*wr(:,:))
      beta = fa2/fa1
      fa1  = fa2
      pr(1:nx-1,1:ny-1) = wr(:,:) + beta*pr(1:nx-1,1:ny-1)
   enddo
   if (it > itmax) then
      ind = -itmax
    else
      ind = it
   endif
   deallocate ( rr, pr, wr, dg )
end subroutine iccg

subroutine ic_solver(rr,wr,dg,nx,ny,mode)
   implicit none
   real rr(nx,ny),wr(nx,ny),dg(nx,ny)
   integer nx,ny,mode,i,j
   if (mode == 1) then
      dg(1,1) = -1/4.0
      do i = 2, nx
         dg(i,1) = -1/(4 + dg(i-1,1))
      enddo
      do j = 2, ny
         dg(1,j) = -1/(4 + dg(1,j-1))
         do i = 2, nx
            dg(i,j) = -1/(4 + dg(i-1,j) + dg(i,j-1))
         enddo
      enddo
   endif
   wr(1,1) = rr(1,1)
   do i = 2, nx
      wr(i,1) = rr(i,1) - wr(i-1,1)*dg(i-1,1)
   enddo
   do j = 2, ny
      wr(1,j) = rr(1,j) - wr(1,j-1)*dg(1,j-1)
      do i = 2, nx
         wr(i,j) = rr(i,j) - wr(i-1,j)*dg(i-1,j) - wr(i,j-1)*dg(i,j-1)
      enddo
   enddo
   wr(nx,ny) = wr(nx,ny)*dg(nx,ny)
   do i = nx-1, 1, -1
      wr(i,ny) = (wr(i,ny) - wr(i+1,ny))*dg(i,ny)
   enddo
   do j = ny-1, 1, -1
      wr(nx,j) = (wr(nx,j) - wr(nx,j+1))*dg(nx,j)
      do i = nx-1, 1, -1
         wr(i,j) = (wr(i,j) - wr(i+1,j) - wr(i,j+1))*dg(i,j)
      enddo
   enddo
end subroutine ic_solver
