!  9.5 ֐ƒZԃt[Gϊ
program chirp_spectrum
   implicit none
   integer, parameter :: nwmax = 256, ntmax = 2000
   real, parameter :: pi = 3.141592653589793, pi2 = 2*pi
   real s0(0:ntmax),s(-nwmax/2:nwmax/2),win(0:nwmax/2)
   real f0,df,t,dt,dts,tsmax,twmax,wid,window
   complex cs(0:ntmax/2)
   integer n,ns,nsmax,nt,nt0,nm,nkmax
   dt  = 0.1
   f0  = 1
   df  = 0.01
   tsmax = ntmax*dt
   nkmax = nwmax/2
   twmax = nwmax*dt
   nsmax = 200
   dts = tsmax/nsmax
   do nt = 0, ntmax
      t = dt*nt
      s0(nt) = sin(pi2*(f0+df*t/2)*t)
   enddo
   wid = 60
   do n = 0, nwmax/2
      win(n)  = window(real(n),wid)
   enddo
   write(10,*) 'signal',nsmax+1,nkmax+1
   do ns = 0, nsmax
      nt0 = dts*ns/dt
      do n = -nwmax/2, nwmax/2
         nm = n + nt0
         if (nm >= 0 .and. nm <= ntmax) then
            s(n) = win(abs(n))*s0(nm)
          else
            s(n) = 0
         endif
      enddo
      call rfft(s,cs,nwmax,-1)
      do n = 0, nkmax
         write(10,*) n,real(ns)*dts,real(n)/twmax,abs(cs(n))**2
      enddo
   enddo
   print *,'Stored Data -> 10'
end program chirp_spectrum

function window(x,w)
   implicit none
   real, parameter :: pi = 3.141592653589793, pi2 = 2*pi
   real window,x,w
   if (abs(x) < w/2) then
      window = 0.5*(1 + cos(pi2*x/w))
    else
      window = 0
   endif
end function window

subroutine rfft(f,cf,n0,mode)
   implicit none
   integer n0,mode,i
   real f(0:n0-1),r1,r2,dn,dn2
   complex cf(0:n0/2),c1,c2,ccr,cci
   integer, save :: ns = 0, n2
   real, parameter :: pi = 3.141592653589793, pi2 = 2*pi
   complex, allocatable, save :: cwp(:)
   real x,dk
   if (ns /= n0) then
      if (mod(n0,2) /= 0) then
         print *,'N0 must be an even number',n0
         return
      endif
      ns = n0
      n2 = n0/2
      if (allocated(cwp)) deallocate ( cwp )
      allocate ( cwp(n2-1) )
      dk = pi2/n0
      do i = 1, n2-1
         x = dk*i
         cwp(i) = cmplx(sin(x), cos(x))
      enddo
   endif
   if (mode < 0) then
      do i = 0, n2-1
         cf(i) = cmplx(f(2*i),f(2*i+1))
      enddo
      call cfft(cf,n2,-2)
      dn  = 1.0/ns
      dn2 = dn/2
      do i = 1, n2/2
         c1 = cf(i)
         c2 = conjg(cf(n2-i))
         ccr = c1 + c2
         cci = c2 - c1
         cf(i) = dn2*(ccr + cci*cwp(i))
         if (i < n2/2) then
            cf(n2-i) = dn2*(conjg(ccr) - conjg(cci)*cwp(n2-i))
         endif
      enddo
      cf(n2) = dn*(real(cf(0)) - imag(cf(0)))
      cf(0)  = dn*(real(cf(0)) + imag(cf(0)))
    else
      r1 = cf(0)
      r2 = cf(n2)
      f(0) = r1 + r2
      f(1) = r1 - r2
      do i = 1, n2/2
         c1 = cf(i)
         c2 = conjg(cf(n2-i))
         ccr = c1 + c2
         cci = c2 - c1
         c1 = ccr + cci*conjg(cwp(i))
         f(2*i)   = real(c1)
         f(2*i+1) = imag(c1)
         if (i < n2/2) then
            c1 = ccr - cci*cwp(n2-i)
            f(2*(n2-i))   = real(c1)
            f(2*(n2-i)+1) = -imag(c1)
         endif
      enddo
      call cfft(f,n2,1)
   endif
end subroutine rfft

subroutine cfft(cf,n0,mode)
   implicit none
   integer n0,mode,i,i1,j,k,n1,n2,ir,it
   complex cf(0:n0-1),c1,c2
   integer, save :: ms, ns = 0, nr
   real, parameter :: pi = 3.141592653589793, pi2 = 2*pi
   complex, allocatable, save :: cwp(:,:)
   integer, allocatable, save :: irev1(:),irev2(:)
   real x,ak
   if (ns /= n0) then
      ms = 0
      ns = n0
      do i = 1, n0
         if (mod(ns,2) /= 0) then
            print *,'N0 must be 2**m',n0
            return
         endif
         ns = ns/2
         ms = ms + 1
         if (ns <= 1) exit
      enddo
      ns = n0
      if (allocated(cwp)) deallocate ( cwp, irev1, irev2 )
      allocate ( cwp(0:n0/2-1,2), irev1(n0/2), irev2(n0/2) )
      ak = pi2/n0
      do i = 0, n0/2-1
         x = ak*i
         cwp(i,1) = cmplx(cos(x), sin(x))
         cwp(i,2) = conjg(cwp(i,1))
      enddo
      nr = 0
      do i = 1, n0-1
         n1 = i
         ir = 0
         do j = 1, ms
            ir = ir*2 + mod(n1,2)
            n1 = n1/2
         enddo
         if (i < ir) then
            nr = nr + 1
            irev1(nr) = i
            irev2(nr) = ir
         endif
      enddo
   endif
   if (mode < 0) then
      it = 2
    else
      it = 1
   endif
   n1 = 1
   n2 = n0
   do k = 1, ms-1
      n2 = n2/2
      do j = 0, n1-1
         do i = 0, n2-1
            i1 = i + j*n2*2
            c1 = cf(i1)
            c2 = cf(i1+n2)
            cf(i1)    = c1 + c2
            cf(i1+n2) = (c1 - c2)*cwp(i*n1,it)
         enddo
      enddo
      n1 = n1*2
   enddo
   do i = 0, n0-1, 2
      c1 = cf(i)
      c2 = cf(i+1)
      cf(i)   = c1 + c2
      cf(i+1) = c1 - c2
   enddo
   do i = 1, nr
      c1    = cf(irev1(i))
      cf(irev1(i)) = cf(irev2(i))
      cf(irev2(i)) = c1
   enddo
   if (mode == 2 .or. mode == -1) cf(:) = cf(:)/ns
end subroutine cfft
