!  10.7 @ɂɑ_̒T
program golden_search_bessel
   implicit none
   real sindx1,xmin,xmax,xpeak(10),ypeak(10),ep
   external sindx1
   integer nmax,nd,i
   xmin = 0
   xmax = 20
   ep = 1e-7
   nd = 10
   nmax = 10
   call golden_search(sindx1,xmin,xmax,nd,xpeak,ypeak,nmax,ep)
   if (nmax > 0) then
      do i = 1, nmax
         print *,'Xpeak, Ypeak = ',xpeak(i),ypeak(i)
      enddo
    else
      print *,'No Peak in the section !',xmin,xmax,nd
   endif
end program golden_search_bessel

function sindx1(x)
   implicit none
   real sindx1,x
   sindx1 = sin(x)/(x+1)
end function sindx1

subroutine golden_search(fun,xmin,xmax,md,xpk,ypk,nmax,eps)
   implicit none
   real fun,xmin,xmax,xpk(*),ypk(*),eps
   real x1,x2,x3,y1,y2,y3,h,xp,yp
   integer md,nmax,ind,i,nm
   external fun
   h = (xmax-xmin)/md
   x1  = xmin;     y1 = fun(x1)
   x2  = x1 + h;   y2 = fun(x2)
   if (nmax <= 0) nmax = md
   nm = 0
   do i = 2, md
      x3 = x2 + h;   y3 = fun(x3)
      if (y3 < y2 .and. y1 < y2) then
         call golden_search_peak(fun,x1,x2,x3,xp,yp,eps,ind)
         if (ind > 0) then
            nm = nm + 1
            xpk(nm) = xp;  ypk(nm) = yp
            if (nm >= nmax) exit
         endif
      endif
      x1 = x2;   y1 = y2
      x2 = x3;   y2 = y3
   enddo
   nmax = nm
end subroutine golden_search

subroutine golden_search_peak(fun,a0,b0,c0,xp,yp,eps,ind)
   implicit none
   real, parameter :: wg = 0.381966011250105
   real fun,a0,b0,c0,xp,yp,eps
   real a,b,c,x,fa,fb,fc,fx
   integer, parameter :: itmax = 1000
   integer ind,it
   a = a0;     fa = fun(a)
   b = b0;     fb = fun(b)
   c = c0;     fc = fun(c)
   x  = a + wg*(c-a)
   fx = fun(x)
   if (fx > fb) then
      b = x;   fb = fx
    else
      a  = (b-wg*c)/(1-wg)
      fa = fun(a)
   endif
   a = a - a0;    b = b - a0;    c = c - a0
   do it = 1, itmax
      x = b + wg*(c-b)
      fx = fun(x+a0)
      if (fx < fb) then
         c = a;   fc = fa
         a = x;   fa = fx
       else
         a = b;   fa = fb
         b = x;   fb = fx
      endif
      if (abs(c-a) < eps*abs(b)) exit
   enddo
   xp = b + a0
   yp = fb
   if (it > itmax) then
      ind = -itmax
    else
      ind = it
   endif
end subroutine golden_search_peak
